=== modified file 'drizzled/functions/Makefile.am'
--- drizzled/functions/Makefile.am	2008-10-29 02:36:17 +0000
+++ drizzled/functions/Makefile.am	2008-10-30 21:40:56 +0000
@@ -67,14 +67,17 @@
 			  real.cc \
 			  round.cc \
 			  row_count.cc \
+			  set_user_var.cc \
 			  shift.cc \
 			  sign.cc \
 			  signed.cc \
 			  sin.cc \
 			  sqrt.cc \
 			  tan.cc \
+			  update_hash.cc \
 			  units.cc \
-			  unsigned.cc
+			  unsigned.cc \
+			  user_var_as_out_param.cc
 
 noinst_HEADERS = abs.h \
 		 acos.h \
@@ -128,12 +131,14 @@
 		 rollup_const.h \
 		 round.h \
 		 row_count.h \
+		 set_user_var.h \
 		 sign.h \
 		 signed.h \
 		 sin.h \
 		 sqrt.h \
 		 tan.h \
-		 unsigned.h
+		 unsigned.h \
+		 user_var_as_out_param.h
 	
 
 DEFS= -DDRIZZLE_SERVER @DEFS@

=== added file 'drizzled/functions/set_user_var.cc'
--- drizzled/functions/set_user_var.cc	1970-01-01 00:00:00 +0000
+++ drizzled/functions/set_user_var.cc	2008-10-30 21:40:56 +0000
@@ -0,0 +1,464 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2008 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; version 2 of the License.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#include <drizzled/server_includes.h>
+#include CSTDINT_H
+#include <drizzled/functions/set_user_var.h>
+
+/*
+  When a user variable is updated (in a SET command or a query like
+  SELECT @a:= ).
+*/
+
+bool Item_func_set_user_var::fix_fields(Session *session, Item **ref)
+{
+  assert(fixed == 0);
+  /* fix_fields will call Item_func_set_user_var::fix_length_and_dec */
+  if (Item_func::fix_fields(session, ref) ||
+      !(entry= get_variable(&session->user_vars, name, 1)))
+    return true;
+  /*
+     Remember the last query which updated it, this way a query can later know
+     if this variable is a constant item in the query (it is if update_query_id
+     is different from query_id).
+  */
+  entry->update_query_id= session->query_id;
+  /*
+    As it is wrong and confusing to associate any
+    character set with NULL, @a should be latin2
+    after this query sequence:
+
+      SET @a=_latin2'string';
+      SET @a=NULL;
+
+    I.e. the second query should not change the charset
+    to the current default value, but should keep the
+    original value assigned during the first query.
+    In order to do it, we don't copy charset
+    from the argument if the argument is NULL
+    and the variable has previously been initialized.
+  */
+  null_item= (args[0]->type() == NULL_ITEM);
+  if (!entry->collation.collation || !null_item)
+    entry->collation.set(args[0]->collation.collation, DERIVATION_IMPLICIT);
+  collation.set(entry->collation.collation, DERIVATION_IMPLICIT);
+  cached_result_type= args[0]->result_type();
+  return false;
+}
+
+void
+Item_func_set_user_var::fix_length_and_dec()
+{
+  maybe_null=args[0]->maybe_null;
+  max_length=args[0]->max_length;
+  decimals=args[0]->decimals;
+  collation.set(args[0]->collation.collation, DERIVATION_IMPLICIT);
+}
+
+/*
+  Mark field in read_map
+
+  NOTES
+    This is used by filesort to register used fields in a a temporary
+    column read set or to register used fields in a view
+*/
+
+bool Item_func_set_user_var::register_field_in_read_map(unsigned char *arg)
+{
+  if (result_field)
+  {
+    Table *table= (Table *) arg;
+    if (result_field->table == table || !table)
+      bitmap_set_bit(result_field->table->read_set, result_field->field_index);
+    if (result_field->vcol_info && result_field->vcol_info->expr_item)
+      return result_field->vcol_info->
+               expr_item->walk(&Item::register_field_in_read_map, 1, arg);
+  }
+  return 0;
+}
+
+/*
+  Mark field in bitmap supplied as *arg
+
+*/
+
+bool Item_func_set_user_var::register_field_in_bitmap(unsigned char *arg)
+{
+  MY_BITMAP *bitmap = (MY_BITMAP *) arg;
+  assert(bitmap);
+  if (result_field)
+  {
+    bitmap_set_bit(bitmap, result_field->field_index);
+  }
+  return 0;
+}
+
+bool
+Item_func_set_user_var::update_hash(void *ptr, uint32_t length,
+                                    Item_result res_type,
+                                    const CHARSET_INFO * const cs, Derivation dv,
+                                    bool unsigned_arg)
+{
+  /*
+    If we set a variable explicitely to NULL then keep the old
+    result type of the variable
+  */
+  if ((null_value= args[0]->null_value) && null_item)
+    res_type= entry->type;                      // Don't change type of item
+  if (::update_hash(entry, (null_value= args[0]->null_value),
+                    ptr, length, res_type, cs, dv, unsigned_arg))
+  {
+    null_value= 1;
+    return 1;
+  }
+  return 0;
+}
+
+/**
+  This functions is invoked on SET \@variable or
+  \@variable:= expression.
+
+  Evaluate (and check expression), store results.
+
+  @note
+    For now it always return OK. All problem with value evaluating
+    will be caught by session->is_error() check in sql_set_variables().
+
+  @retval
+    false OK.
+*/
+
+bool
+Item_func_set_user_var::check(bool use_result_field)
+{
+  if (use_result_field && !result_field)
+    use_result_field= false;
+
+  switch (cached_result_type) {
+  case REAL_RESULT:
+  {
+    save_result.vreal= use_result_field ? result_field->val_real() :
+                        args[0]->val_real();
+    break;
+  }
+  case INT_RESULT:
+  {
+    save_result.vint= use_result_field ? result_field->val_int() :
+                       args[0]->val_int();
+    unsigned_flag= use_result_field ? ((Field_num*)result_field)->unsigned_flag:
+                    args[0]->unsigned_flag;
+    break;
+  }
+  case STRING_RESULT:
+  {
+    save_result.vstr= use_result_field ? result_field->val_str(&value) :
+                       args[0]->val_str(&value);
+    break;
+  }
+  case DECIMAL_RESULT:
+  {
+    save_result.vdec= use_result_field ?
+                       result_field->val_decimal(&decimal_buff) :
+                       args[0]->val_decimal(&decimal_buff);
+    break;
+  }
+  case ROW_RESULT:
+  default:
+    // This case should never be chosen
+    assert(0);
+    break;
+  }
+  return(false);
+}
+
+/**
+  This functions is invoked on
+  SET \@variable or \@variable:= expression.
+
+  @note
+    We have to store the expression as such in the variable, independent of
+    the value method used by the user
+
+  @retval
+    0   OK
+  @retval
+    1   EOM Error
+
+*/
+
+bool
+Item_func_set_user_var::update()
+{
+  bool res= false;
+
+  switch (cached_result_type) {
+  case REAL_RESULT:
+  {
+    res= update_hash((void*) &save_result.vreal,sizeof(save_result.vreal),
+                     REAL_RESULT, &my_charset_bin, DERIVATION_IMPLICIT, 0);
+    break;
+  }
+  case INT_RESULT:
+  {
+    res= update_hash((void*) &save_result.vint, sizeof(save_result.vint),
+                     INT_RESULT, &my_charset_bin, DERIVATION_IMPLICIT,
+                     unsigned_flag);
+    break;
+  }
+  case STRING_RESULT:
+  {
+    if (!save_result.vstr)                                      // Null value
+      res= update_hash((void*) 0, 0, STRING_RESULT, &my_charset_bin,
+                       DERIVATION_IMPLICIT, 0);
+    else
+      res= update_hash((void*) save_result.vstr->ptr(),
+                       save_result.vstr->length(), STRING_RESULT,
+                       save_result.vstr->charset(),
+                       DERIVATION_IMPLICIT, 0);
+    break;
+  }
+  case DECIMAL_RESULT:
+  {
+    if (!save_result.vdec)                                      // Null value
+      res= update_hash((void*) 0, 0, DECIMAL_RESULT, &my_charset_bin,
+                       DERIVATION_IMPLICIT, 0);
+    else
+      res= update_hash((void*) save_result.vdec,
+                       sizeof(my_decimal), DECIMAL_RESULT,
+                       &my_charset_bin, DERIVATION_IMPLICIT, 0);
+    break;
+  }
+  case ROW_RESULT:
+  default:
+    // This case should never be chosen
+    assert(0);
+    break;
+  }
+  return(res);
+}
+
+double Item_func_set_user_var::val_real()
+{
+  assert(fixed == 1);
+  check(0);
+  update();                                     // Store expression
+  return entry->val_real(&null_value);
+}
+
+int64_t Item_func_set_user_var::val_int()
+{
+  assert(fixed == 1);
+  check(0);
+  update();                                     // Store expression
+  return entry->val_int(&null_value);
+}
+
+String *Item_func_set_user_var::val_str(String *str)
+{
+  assert(fixed == 1);
+  check(0);
+  update();                                     // Store expression
+  return entry->val_str(&null_value, str, decimals);
+}
+
+
+my_decimal *Item_func_set_user_var::val_decimal(my_decimal *val)
+{
+  assert(fixed == 1);
+  check(0);
+  update();                                     // Store expression
+  return entry->val_decimal(&null_value, val);
+}
+
+double Item_func_set_user_var::val_result()
+{
+  assert(fixed == 1);
+  check(true);
+  update();                                     // Store expression
+  return entry->val_real(&null_value);
+}
+
+int64_t Item_func_set_user_var::val_int_result()
+{
+  assert(fixed == 1);
+  check(true);
+  update();                                     // Store expression
+  return entry->val_int(&null_value);
+}
+
+String *Item_func_set_user_var::str_result(String *str)
+{
+  assert(fixed == 1);
+  check(true);
+  update();                                     // Store expression
+  return entry->val_str(&null_value, str, decimals);
+}
+
+
+my_decimal *Item_func_set_user_var::val_decimal_result(my_decimal *val)
+{
+  assert(fixed == 1);
+  check(true);
+  update();                                     // Store expression
+  return entry->val_decimal(&null_value, val);
+}
+
+void Item_func_set_user_var::print(String *str, enum_query_type query_type)
+{
+  str->append(STRING_WITH_LEN("(@"));
+  str->append(name.str, name.length);
+  str->append(STRING_WITH_LEN(":="));
+  args[0]->print(str, query_type);
+  str->append(')');
+}
+
+
+void Item_func_set_user_var::print_as_stmt(String *str,
+                                           enum_query_type query_type)
+{
+  str->append(STRING_WITH_LEN("set @"));
+  str->append(name.str, name.length);
+  str->append(STRING_WITH_LEN(":="));
+  args[0]->print(str, query_type);
+  str->append(')');
+}
+
+bool Item_func_set_user_var::send(Protocol *protocol, String *str_arg)
+{
+  if (result_field)
+  {
+    check(1);
+    update();
+    return protocol->store(result_field);
+  }
+  return Item::send(protocol, str_arg);
+}
+
+void Item_func_set_user_var::make_field(Send_field *tmp_field)
+{
+  if (result_field)
+  {
+    result_field->make_field(tmp_field);
+    assert(tmp_field->table_name != 0);
+    if (Item::name)
+      tmp_field->col_name=Item::name;               // Use user supplied name
+  }
+  else
+    Item::make_field(tmp_field);
+}
+
+/*
+  Save the value of a user variable into a field
+
+  SYNOPSIS
+    save_in_field()
+      field           target field to save the value to
+      no_conversion   flag indicating whether conversions are allowed
+
+  DESCRIPTION
+    Save the function value into a field and update the user variable
+    accordingly. If a result field is defined and the target field doesn't
+    coincide with it then the value from the result field will be used as
+    the new value of the user variable.
+
+    The reason to have this method rather than simply using the result
+    field in the val_xxx() methods is that the value from the result field
+    not always can be used when the result field is defined.
+    Let's consider the following cases:
+    1) when filling a tmp table the result field is defined but the value of it
+    is undefined because it has to be produced yet. Thus we can't use it.
+    2) on execution of an INSERT ... SELECT statement the save_in_field()
+    function will be called to fill the data in the new record. If the SELECT
+    part uses a tmp table then the result field is defined and should be
+    used in order to get the correct result.
+
+    The difference between the SET_USER_VAR function and regular functions
+    like CONCAT is that the Item_func objects for the regular functions are
+    replaced by Item_field objects after the values of these functions have
+    been stored in a tmp table. Yet an object of the Item_field class cannot
+    be used to update a user variable.
+    Due to this we have to handle the result field in a special way here and
+    in the Item_func_set_user_var::send() function.
+
+  RETURN VALUES
+    false       Ok
+    true        Error
+*/
+
+int Item_func_set_user_var::save_in_field(Field *field, bool no_conversions,
+                                          bool can_use_result_field)
+{
+  bool use_result_field= (!can_use_result_field ? 0 :
+                          (result_field && result_field != field));
+  int error;
+
+  /* Update the value of the user variable */
+  check(use_result_field);
+  update();
+
+  if (result_type() == STRING_RESULT ||
+      (result_type() == REAL_RESULT && field->result_type() == STRING_RESULT))
+  {
+    String *result;
+    const CHARSET_INFO * const cs= collation.collation;
+    char buff[MAX_FIELD_WIDTH];         // Alloc buffer for small columns
+    str_value.set_quick(buff, sizeof(buff), cs);
+    result= entry->val_str(&null_value, &str_value, decimals);
+
+    if (null_value)
+    {
+      str_value.set_quick(0, 0, cs);
+      return set_field_to_null_with_conversions(field, no_conversions);
+    }
+
+    /* NOTE: If null_value == false, "result" must be not NULL.  */
+
+    field->set_notnull();
+    error=field->store(result->ptr(),result->length(),cs);
+    str_value.set_quick(0, 0, cs);
+  }
+  else if (result_type() == REAL_RESULT)
+  {
+    double nr= entry->val_real(&null_value);
+    if (null_value)
+      return set_field_to_null(field);
+    field->set_notnull();
+    error=field->store(nr);
+  }
+  else if (result_type() == DECIMAL_RESULT)
+  {
+    my_decimal decimal_value;
+    my_decimal *val= entry->val_decimal(&null_value, &decimal_value);
+    if (null_value)
+      return set_field_to_null(field);
+    field->set_notnull();
+    error=field->store_decimal(val);
+  }
+  else
+  {
+    int64_t nr= entry->val_int(&null_value);
+    if (null_value)
+      return set_field_to_null_with_conversions(field, no_conversions);
+    field->set_notnull();
+    error=field->store(nr, unsigned_flag);
+  }
+  return error;
+}
+
+

=== added file 'drizzled/functions/set_user_var.h'
--- drizzled/functions/set_user_var.h	1970-01-01 00:00:00 +0000
+++ drizzled/functions/set_user_var.h	2008-10-30 21:40:56 +0000
@@ -0,0 +1,82 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2008 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; version 2 of the License.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef DRIZZLED_FUNCTIONS_SET_USER_VAR_H
+#define DRIZZLED_FUNCTIONS_SET_USER_VAR_H
+
+#include <drizzled/functions/func.h> 
+
+/* Handling of user definable variables */
+
+class user_var_entry;
+
+class Item_func_set_user_var :public Item_func
+{
+  enum Item_result cached_result_type;
+  user_var_entry *entry;
+  char buffer[MAX_FIELD_WIDTH];
+  String value;
+  my_decimal decimal_buff;
+  bool null_item;
+  union
+  {
+    int64_t vint;
+    double vreal;
+    String *vstr;
+    my_decimal *vdec;
+  } save_result;
+
+public:
+  LEX_STRING name; // keep it public
+  Item_func_set_user_var(LEX_STRING a,Item *b)
+    :Item_func(b), cached_result_type(INT_RESULT), name(a)
+  {}
+  enum Functype functype() const { return SUSERVAR_FUNC; }
+  double val_real();
+  int64_t val_int();
+  String *val_str(String *str);
+  my_decimal *val_decimal(my_decimal *);
+  double val_result();
+  int64_t val_int_result();
+  String *str_result(String *str);
+  my_decimal *val_decimal_result(my_decimal *);
+  bool update_hash(void *ptr, uint32_t length, enum Item_result type,
+  		   const CHARSET_INFO * const cs, Derivation dv, bool unsigned_arg);
+  bool send(Protocol *protocol, String *str_arg);
+  void make_field(Send_field *tmp_field);
+  bool check(bool use_result_field);
+  bool update();
+  enum Item_result result_type () const { return cached_result_type; }
+  bool fix_fields(Session *session, Item **ref);
+  void fix_length_and_dec();
+  virtual void print(String *str, enum_query_type query_type);
+  void print_as_stmt(String *str, enum_query_type query_type);
+  const char *func_name() const { return "set_user_var"; }
+  int save_in_field(Field *field, bool no_conversions,
+                    bool can_use_result_field);
+  int save_in_field(Field *field, bool no_conversions)
+  {
+    return save_in_field(field, no_conversions, 1);
+  }
+  void save_org_in_field(Field *field) { (void)save_in_field(field, 1, 0); }
+  bool register_field_in_read_map(unsigned char *arg);
+  bool register_field_in_bitmap(unsigned char *arg);
+};
+
+#endif /* DRIZZLED_FUNCTIONS_SET_USER_VAR_H */

=== added file 'drizzled/functions/update_hash.cc'
--- drizzled/functions/update_hash.cc	1970-01-01 00:00:00 +0000
+++ drizzled/functions/update_hash.cc	2008-10-30 21:40:56 +0000
@@ -0,0 +1,103 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2008 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; version 2 of the License.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#include <drizzled/server_includes.h>
+#include CSTDINT_H
+#include <drizzled/functions/update_hash.h>
+
+/**
+  Set value to user variable.
+
+  @param entry          pointer to structure representing variable
+  @param set_null       should we set NULL value ?
+  @param ptr            pointer to buffer with new value
+  @param length         length of new value
+  @param type           type of new value
+  @param cs             charset info for new value
+  @param dv             derivation for new value
+  @param unsigned_arg   indiates if a value of type INT_RESULT is unsigned
+
+  @note Sets error and fatal error if allocation fails.
+
+  @retval
+    false   success
+  @retval
+    true    failure
+*/
+
+#define extra_size sizeof(double)
+
+bool
+update_hash(user_var_entry *entry, bool set_null, void *ptr, uint32_t length,
+            Item_result type, const CHARSET_INFO * const cs, Derivation dv,
+            bool unsigned_arg)
+{
+  if (set_null)
+  {
+    char *pos= (char*) entry+ ALIGN_SIZE(sizeof(user_var_entry));
+    if (entry->value && entry->value != pos)
+      free(entry->value);
+    entry->value= 0;
+    entry->length= 0;
+  }
+  else
+  {
+    if (type == STRING_RESULT)
+      length++;					// Store strings with end \0
+    if (length <= extra_size)
+    {
+      /* Save value in value struct */
+      char *pos= (char*) entry+ ALIGN_SIZE(sizeof(user_var_entry));
+      if (entry->value != pos)
+      {
+	if (entry->value)
+	  free(entry->value);
+	entry->value=pos;
+      }
+    }
+    else
+    {
+      /* Allocate variable */
+      if (entry->length != length)
+      {
+	char *pos= (char*) entry+ ALIGN_SIZE(sizeof(user_var_entry));
+	if (entry->value == pos)
+	  entry->value=0;
+        entry->value= (char*) my_realloc(entry->value, length,
+                                         MYF(MY_ALLOW_ZERO_PTR | MY_WME |
+                                             ME_FATALERROR));
+        if (!entry->value)
+	  return 1;
+      }
+    }
+    if (type == STRING_RESULT)
+    {
+      length--;					// Fix length change above
+      entry->value[length]= 0;			// Store end \0
+    }
+    memcpy(entry->value,ptr,length);
+    if (type == DECIMAL_RESULT)
+      ((my_decimal*)entry->value)->fix_buffer_pointer();
+    entry->length= length;
+    entry->collation.set(cs, dv);
+    entry->unsigned_flag= unsigned_arg;
+  }
+  entry->type=type;
+  return 0;
+}

=== added file 'drizzled/functions/update_hash.h'
--- drizzled/functions/update_hash.h	1970-01-01 00:00:00 +0000
+++ drizzled/functions/update_hash.h	2008-10-30 21:40:56 +0000
@@ -0,0 +1,38 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2008 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; version 2 of the License.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef DRIZZLED_FUNCTIONS_UPDATE_HASH_H
+#define DRIZZLED_FUNCTIONS_UPDATE_HASH_H
+
+#include <drizzled/functions/func.h> 
+
+bool
+update_hash(
+  user_var_entry *entry, 
+  bool set_null, 
+  void *ptr, 
+  uint32_t length,
+  Item_result type, 
+  const CHARSET_INFO * const cs, 
+  Derivation dv,
+  bool unsigned_arg
+  );
+
+
+#endif /* DRIZZLED_FUNCTIONS_UPDATE_HASH_H */

=== added file 'drizzled/functions/user_var_as_out_param.cc'
--- drizzled/functions/user_var_as_out_param.cc	1970-01-01 00:00:00 +0000
+++ drizzled/functions/user_var_as_out_param.cc	2008-10-30 21:40:56 +0000
@@ -0,0 +1,88 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2008 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; version 2 of the License.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#include <drizzled/server_includes.h>
+#include CSTDINT_H
+#include <drizzled/functions/user_var_as_out_param.h>
+
+bool Item_user_var_as_out_param::fix_fields(Session *session, Item **ref)
+{
+  assert(fixed == 0);
+  if (Item::fix_fields(session, ref) ||
+      !(entry= get_variable(&session->user_vars, name, 1)))
+    return true;
+  entry->type= STRING_RESULT;
+  /*
+    Let us set the same collation which is used for loading
+    of fields in LOAD DATA INFILE.
+    (Since Item_user_var_as_out_param is used only there).
+  */
+  entry->collation.set(session->variables.collation_database);
+  entry->update_query_id= session->query_id;
+  return false;
+}
+
+void Item_user_var_as_out_param::set_null_value(const CHARSET_INFO * const cs)
+{
+  ::update_hash(entry, true, 0, 0, STRING_RESULT, cs,
+                DERIVATION_IMPLICIT, 0 /* unsigned_arg */);
+}
+
+
+void Item_user_var_as_out_param::set_value(const char *str, uint32_t length,
+                                           const CHARSET_INFO * const cs)
+{
+  ::update_hash(entry, false, (void*)str, length, STRING_RESULT, cs,
+                DERIVATION_IMPLICIT, 0 /* unsigned_arg */);
+}
+
+double Item_user_var_as_out_param::val_real()
+{
+  assert(0);
+  return 0.0;
+}
+
+
+int64_t Item_user_var_as_out_param::val_int()
+{
+  assert(0);
+  return 0;
+}
+
+
+String* Item_user_var_as_out_param::val_str(String *str __attribute__((unused)))
+{
+  assert(0);
+  return 0;
+}
+
+my_decimal* Item_user_var_as_out_param::val_decimal(my_decimal *decimal_buffer __attribute__((unused)))
+{
+  assert(0);
+  return 0;
+}
+
+
+void Item_user_var_as_out_param::print(String *str,
+                                       enum_query_type query_type __attribute__((unused)))
+{
+  str->append('@');
+  str->append(name.str,name.length);
+}
+

=== added file 'drizzled/functions/user_var_as_out_param.h'
--- drizzled/functions/user_var_as_out_param.h	1970-01-01 00:00:00 +0000
+++ drizzled/functions/user_var_as_out_param.h	2008-10-30 21:40:56 +0000
@@ -0,0 +1,53 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2008 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; version 2 of the License.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef DRIZZLED_FUNCTIONS_USER_VAR_AS_OUT_PARAM_H
+#define DRIZZLED_FUNCTIONS_USER_VAR_AS_OUT_PARAM_H
+
+#include <drizzled/functions/func.h> 
+
+/*
+  This item represents user variable used as out parameter (e.g in LOAD DATA),
+  and it is supposed to be used only for this purprose. So it is simplified
+  a lot. Actually you should never obtain its value.
+
+  The only two reasons for this thing being an Item is possibility to store it
+  in List<Item> and desire to place this code somewhere near other functions
+  working with user variables.
+*/
+class Item_user_var_as_out_param :public Item
+{
+  LEX_STRING name;
+  user_var_entry *entry;
+public:
+  Item_user_var_as_out_param(LEX_STRING a) : name(a) {}
+  /* We should return something different from FIELD_ITEM here */
+  enum Type type() const { return STRING_ITEM;}
+  double val_real();
+  int64_t val_int();
+  String *val_str(String *str);
+  my_decimal *val_decimal(my_decimal *decimal_buffer);
+  /* fix_fields() binds variable name with its entry structure */
+  bool fix_fields(Session *session, Item **ref);
+  virtual void print(String *str, enum_query_type query_type);
+  void set_null_value(const CHARSET_INFO * const cs);
+  void set_value(const char *str, uint32_t length, const CHARSET_INFO * const cs);
+};
+
+#endif /* DRIZZLED_FUNCTIONS_USER_VAR_AS_OUT_PARAM_H */

=== modified file 'drizzled/item_func.cc'
--- drizzled/item_func.cc	2008-10-30 02:40:49 +0000
+++ drizzled/item_func.cc	2008-10-30 21:40:56 +0000
@@ -150,201 +150,6 @@
   return copy_or_same(session);
 }
 
-/*
-  When a user variable is updated (in a SET command or a query like
-  SELECT @a:= ).
-*/
-
-bool Item_func_set_user_var::fix_fields(Session *session, Item **ref)
-{
-  assert(fixed == 0);
-  /* fix_fields will call Item_func_set_user_var::fix_length_and_dec */
-  if (Item_func::fix_fields(session, ref) ||
-      !(entry= get_variable(&session->user_vars, name, 1)))
-    return true;
-  /* 
-     Remember the last query which updated it, this way a query can later know
-     if this variable is a constant item in the query (it is if update_query_id
-     is different from query_id).
-  */
-  entry->update_query_id= session->query_id;
-  /*
-    As it is wrong and confusing to associate any 
-    character set with NULL, @a should be latin2
-    after this query sequence:
-
-      SET @a=_latin2'string';
-      SET @a=NULL;
-
-    I.e. the second query should not change the charset
-    to the current default value, but should keep the 
-    original value assigned during the first query.
-    In order to do it, we don't copy charset
-    from the argument if the argument is NULL
-    and the variable has previously been initialized.
-  */
-  null_item= (args[0]->type() == NULL_ITEM);
-  if (!entry->collation.collation || !null_item)
-    entry->collation.set(args[0]->collation.collation, DERIVATION_IMPLICIT);
-  collation.set(entry->collation.collation, DERIVATION_IMPLICIT);
-  cached_result_type= args[0]->result_type();
-  return false;
-}
-
-
-void
-Item_func_set_user_var::fix_length_and_dec()
-{
-  maybe_null=args[0]->maybe_null;
-  max_length=args[0]->max_length;
-  decimals=args[0]->decimals;
-  collation.set(args[0]->collation.collation, DERIVATION_IMPLICIT);
-}
-
-
-/*
-  Mark field in read_map
-
-  NOTES
-    This is used by filesort to register used fields in a a temporary
-    column read set or to register used fields in a view
-*/
-
-bool Item_func_set_user_var::register_field_in_read_map(unsigned char *arg)
-{
-  if (result_field)
-  {
-    Table *table= (Table *) arg;
-    if (result_field->table == table || !table)
-      bitmap_set_bit(result_field->table->read_set, result_field->field_index);
-    if (result_field->vcol_info && result_field->vcol_info->expr_item)
-      return result_field->vcol_info->
-               expr_item->walk(&Item::register_field_in_read_map, 1, arg);
-  }
-  return 0;
-}
-
-
-/*
-  Mark field in bitmap supplied as *arg
-
-*/
-
-bool Item_func_set_user_var::register_field_in_bitmap(unsigned char *arg)
-{
-  MY_BITMAP *bitmap = (MY_BITMAP *) arg;
-  assert(bitmap);
-  if (result_field)
-  {
-    bitmap_set_bit(bitmap, result_field->field_index);
-  }
-  return 0;
-}
-
-
-/**
-  Set value to user variable.
-
-  @param entry          pointer to structure representing variable
-  @param set_null       should we set NULL value ?
-  @param ptr            pointer to buffer with new value
-  @param length         length of new value
-  @param type           type of new value
-  @param cs             charset info for new value
-  @param dv             derivation for new value
-  @param unsigned_arg   indiates if a value of type INT_RESULT is unsigned
-
-  @note Sets error and fatal error if allocation fails.
-
-  @retval
-    false   success
-  @retval
-    true    failure
-*/
-
-#define extra_size sizeof(double)
-
-static bool
-update_hash(user_var_entry *entry, bool set_null, void *ptr, uint32_t length,
-            Item_result type, const CHARSET_INFO * const cs, Derivation dv,
-            bool unsigned_arg)
-{
-  if (set_null)
-  {
-    char *pos= (char*) entry+ ALIGN_SIZE(sizeof(user_var_entry));
-    if (entry->value && entry->value != pos)
-      free(entry->value);
-    entry->value= 0;
-    entry->length= 0;
-  }
-  else
-  {
-    if (type == STRING_RESULT)
-      length++;					// Store strings with end \0
-    if (length <= extra_size)
-    {
-      /* Save value in value struct */
-      char *pos= (char*) entry+ ALIGN_SIZE(sizeof(user_var_entry));
-      if (entry->value != pos)
-      {
-	if (entry->value)
-	  free(entry->value);
-	entry->value=pos;
-      }
-    }
-    else
-    {
-      /* Allocate variable */
-      if (entry->length != length)
-      {
-	char *pos= (char*) entry+ ALIGN_SIZE(sizeof(user_var_entry));
-	if (entry->value == pos)
-	  entry->value=0;
-        entry->value= (char*) my_realloc(entry->value, length,
-                                         MYF(MY_ALLOW_ZERO_PTR | MY_WME |
-                                             ME_FATALERROR));
-        if (!entry->value)
-	  return 1;
-      }
-    }
-    if (type == STRING_RESULT)
-    {
-      length--;					// Fix length change above
-      entry->value[length]= 0;			// Store end \0
-    }
-    memcpy(entry->value,ptr,length);
-    if (type == DECIMAL_RESULT)
-      ((my_decimal*)entry->value)->fix_buffer_pointer();
-    entry->length= length;
-    entry->collation.set(cs, dv);
-    entry->unsigned_flag= unsigned_arg;
-  }
-  entry->type=type;
-  return 0;
-}
-
-
-bool
-Item_func_set_user_var::update_hash(void *ptr, uint32_t length,
-                                    Item_result res_type,
-                                    const CHARSET_INFO * const cs, Derivation dv,
-                                    bool unsigned_arg)
-{
-  /*
-    If we set a variable explicitely to NULL then keep the old
-    result type of the variable
-  */
-  if ((null_value= args[0]->null_value) && null_item)
-    res_type= entry->type;                      // Don't change type of item
-  if (::update_hash(entry, (null_value= args[0]->null_value),
-                    ptr, length, res_type, cs, dv, unsigned_arg))
-  {
-    null_value= 1;
-    return 1;
-  }
-  return 0;
-}
-
 
 /** Get the value of a variable as a double. */
 
@@ -462,428 +267,3 @@
   }
   return(val);
 }
-
-/**
-  This functions is invoked on SET \@variable or
-  \@variable:= expression.
-
-  Evaluate (and check expression), store results.
-
-  @note
-    For now it always return OK. All problem with value evaluating
-    will be caught by session->is_error() check in sql_set_variables().
-
-  @retval
-    false OK.
-*/
-
-bool
-Item_func_set_user_var::check(bool use_result_field)
-{
-  if (use_result_field && !result_field)
-    use_result_field= false;
-
-  switch (cached_result_type) {
-  case REAL_RESULT:
-  {
-    save_result.vreal= use_result_field ? result_field->val_real() :
-                        args[0]->val_real();
-    break;
-  }
-  case INT_RESULT:
-  {
-    save_result.vint= use_result_field ? result_field->val_int() :
-                       args[0]->val_int();
-    unsigned_flag= use_result_field ? ((Field_num*)result_field)->unsigned_flag:
-                    args[0]->unsigned_flag;
-    break;
-  }
-  case STRING_RESULT:
-  {
-    save_result.vstr= use_result_field ? result_field->val_str(&value) :
-                       args[0]->val_str(&value);
-    break;
-  }
-  case DECIMAL_RESULT:
-  {
-    save_result.vdec= use_result_field ?
-                       result_field->val_decimal(&decimal_buff) :
-                       args[0]->val_decimal(&decimal_buff);
-    break;
-  }
-  case ROW_RESULT:
-  default:
-    // This case should never be chosen
-    assert(0);
-    break;
-  }
-  return(false);
-}
-
-
-/**
-  This functions is invoked on
-  SET \@variable or \@variable:= expression.
-
-  @note
-    We have to store the expression as such in the variable, independent of
-    the value method used by the user
-
-  @retval
-    0	OK
-  @retval
-    1	EOM Error
-
-*/
-
-bool
-Item_func_set_user_var::update()
-{
-  bool res= false;
-
-  switch (cached_result_type) {
-  case REAL_RESULT:
-  {
-    res= update_hash((void*) &save_result.vreal,sizeof(save_result.vreal),
-		     REAL_RESULT, &my_charset_bin, DERIVATION_IMPLICIT, 0);
-    break;
-  }
-  case INT_RESULT:
-  {
-    res= update_hash((void*) &save_result.vint, sizeof(save_result.vint),
-                     INT_RESULT, &my_charset_bin, DERIVATION_IMPLICIT,
-                     unsigned_flag);
-    break;
-  }
-  case STRING_RESULT:
-  {
-    if (!save_result.vstr)					// Null value
-      res= update_hash((void*) 0, 0, STRING_RESULT, &my_charset_bin,
-		       DERIVATION_IMPLICIT, 0);
-    else
-      res= update_hash((void*) save_result.vstr->ptr(),
-		       save_result.vstr->length(), STRING_RESULT,
-		       save_result.vstr->charset(),
-		       DERIVATION_IMPLICIT, 0);
-    break;
-  }
-  case DECIMAL_RESULT:
-  {
-    if (!save_result.vdec)					// Null value
-      res= update_hash((void*) 0, 0, DECIMAL_RESULT, &my_charset_bin,
-                       DERIVATION_IMPLICIT, 0);
-    else
-      res= update_hash((void*) save_result.vdec,
-                       sizeof(my_decimal), DECIMAL_RESULT,
-                       &my_charset_bin, DERIVATION_IMPLICIT, 0);
-    break;
-  }
-  case ROW_RESULT:
-  default:
-    // This case should never be chosen
-    assert(0);
-    break;
-  }
-  return(res);
-}
-
-
-double Item_func_set_user_var::val_real()
-{
-  assert(fixed == 1);
-  check(0);
-  update();					// Store expression
-  return entry->val_real(&null_value);
-}
-
-int64_t Item_func_set_user_var::val_int()
-{
-  assert(fixed == 1);
-  check(0);
-  update();					// Store expression
-  return entry->val_int(&null_value);
-}
-
-String *Item_func_set_user_var::val_str(String *str)
-{
-  assert(fixed == 1);
-  check(0);
-  update();					// Store expression
-  return entry->val_str(&null_value, str, decimals);
-}
-
-
-my_decimal *Item_func_set_user_var::val_decimal(my_decimal *val)
-{
-  assert(fixed == 1);
-  check(0);
-  update();					// Store expression
-  return entry->val_decimal(&null_value, val);
-}
-
-
-double Item_func_set_user_var::val_result()
-{
-  assert(fixed == 1);
-  check(true);
-  update();					// Store expression
-  return entry->val_real(&null_value);
-}
-
-int64_t Item_func_set_user_var::val_int_result()
-{
-  assert(fixed == 1);
-  check(true);
-  update();					// Store expression
-  return entry->val_int(&null_value);
-}
-
-String *Item_func_set_user_var::str_result(String *str)
-{
-  assert(fixed == 1);
-  check(true);
-  update();					// Store expression
-  return entry->val_str(&null_value, str, decimals);
-}
-
-
-my_decimal *Item_func_set_user_var::val_decimal_result(my_decimal *val)
-{
-  assert(fixed == 1);
-  check(true);
-  update();					// Store expression
-  return entry->val_decimal(&null_value, val);
-}
-
-
-void Item_func_set_user_var::print(String *str, enum_query_type query_type)
-{
-  str->append(STRING_WITH_LEN("(@"));
-  str->append(name.str, name.length);
-  str->append(STRING_WITH_LEN(":="));
-  args[0]->print(str, query_type);
-  str->append(')');
-}
-
-
-void Item_func_set_user_var::print_as_stmt(String *str,
-                                           enum_query_type query_type)
-{
-  str->append(STRING_WITH_LEN("set @"));
-  str->append(name.str, name.length);
-  str->append(STRING_WITH_LEN(":="));
-  args[0]->print(str, query_type);
-  str->append(')');
-}
-
-bool Item_func_set_user_var::send(Protocol *protocol, String *str_arg)
-{
-  if (result_field)
-  {
-    check(1);
-    update();
-    return protocol->store(result_field);
-  }
-  return Item::send(protocol, str_arg);
-}
-
-void Item_func_set_user_var::make_field(Send_field *tmp_field)
-{
-  if (result_field)
-  {
-    result_field->make_field(tmp_field);
-    assert(tmp_field->table_name != 0);
-    if (Item::name)
-      tmp_field->col_name=Item::name;               // Use user supplied name
-  }
-  else
-    Item::make_field(tmp_field);
-}
-
-
-/*
-  Save the value of a user variable into a field
-
-  SYNOPSIS
-    save_in_field()
-      field           target field to save the value to
-      no_conversion   flag indicating whether conversions are allowed
-
-  DESCRIPTION
-    Save the function value into a field and update the user variable
-    accordingly. If a result field is defined and the target field doesn't
-    coincide with it then the value from the result field will be used as
-    the new value of the user variable.
-
-    The reason to have this method rather than simply using the result
-    field in the val_xxx() methods is that the value from the result field
-    not always can be used when the result field is defined.
-    Let's consider the following cases:
-    1) when filling a tmp table the result field is defined but the value of it
-    is undefined because it has to be produced yet. Thus we can't use it.
-    2) on execution of an INSERT ... SELECT statement the save_in_field()
-    function will be called to fill the data in the new record. If the SELECT
-    part uses a tmp table then the result field is defined and should be
-    used in order to get the correct result.
-
-    The difference between the SET_USER_VAR function and regular functions
-    like CONCAT is that the Item_func objects for the regular functions are
-    replaced by Item_field objects after the values of these functions have
-    been stored in a tmp table. Yet an object of the Item_field class cannot
-    be used to update a user variable.
-    Due to this we have to handle the result field in a special way here and
-    in the Item_func_set_user_var::send() function.
-
-  RETURN VALUES
-    false       Ok
-    true        Error
-*/
-
-int Item_func_set_user_var::save_in_field(Field *field, bool no_conversions,
-                                          bool can_use_result_field)
-{
-  bool use_result_field= (!can_use_result_field ? 0 :
-                          (result_field && result_field != field));
-  int error;
-
-  /* Update the value of the user variable */
-  check(use_result_field);
-  update();
-
-  if (result_type() == STRING_RESULT ||
-      (result_type() == REAL_RESULT && field->result_type() == STRING_RESULT))
-  {
-    String *result;
-    const CHARSET_INFO * const cs= collation.collation;
-    char buff[MAX_FIELD_WIDTH];		// Alloc buffer for small columns
-    str_value.set_quick(buff, sizeof(buff), cs);
-    result= entry->val_str(&null_value, &str_value, decimals);
-
-    if (null_value)
-    {
-      str_value.set_quick(0, 0, cs);
-      return set_field_to_null_with_conversions(field, no_conversions);
-    }
-
-    /* NOTE: If null_value == false, "result" must be not NULL.  */
-
-    field->set_notnull();
-    error=field->store(result->ptr(),result->length(),cs);
-    str_value.set_quick(0, 0, cs);
-  }
-  else if (result_type() == REAL_RESULT)
-  {
-    double nr= entry->val_real(&null_value);
-    if (null_value)
-      return set_field_to_null(field);
-    field->set_notnull();
-    error=field->store(nr);
-  }
-  else if (result_type() == DECIMAL_RESULT)
-  {
-    my_decimal decimal_value;
-    my_decimal *val= entry->val_decimal(&null_value, &decimal_value);
-    if (null_value)
-      return set_field_to_null(field);
-    field->set_notnull();
-    error=field->store_decimal(val);
-  }
-  else
-  {
-    int64_t nr= entry->val_int(&null_value);
-    if (null_value)
-      return set_field_to_null_with_conversions(field, no_conversions);
-    field->set_notnull();
-    error=field->store(nr, unsigned_flag);
-  }
-  return error;
-}
-
-
-bool Item_user_var_as_out_param::fix_fields(Session *session, Item **ref)
-{
-  assert(fixed == 0);
-  if (Item::fix_fields(session, ref) ||
-      !(entry= get_variable(&session->user_vars, name, 1)))
-    return true;
-  entry->type= STRING_RESULT;
-  /*
-    Let us set the same collation which is used for loading
-    of fields in LOAD DATA INFILE.
-    (Since Item_user_var_as_out_param is used only there).
-  */
-  entry->collation.set(session->variables.collation_database);
-  entry->update_query_id= session->query_id;
-  return false;
-}
-
-
-void Item_user_var_as_out_param::set_null_value(const CHARSET_INFO * const cs)
-{
-  ::update_hash(entry, true, 0, 0, STRING_RESULT, cs,
-                DERIVATION_IMPLICIT, 0 /* unsigned_arg */);
-}
-
-
-void Item_user_var_as_out_param::set_value(const char *str, uint32_t length,
-                                           const CHARSET_INFO * const cs)
-{
-  ::update_hash(entry, false, (void*)str, length, STRING_RESULT, cs,
-                DERIVATION_IMPLICIT, 0 /* unsigned_arg */);
-}
-
-
-double Item_user_var_as_out_param::val_real()
-{
-  assert(0);
-  return 0.0;
-}
-
-
-int64_t Item_user_var_as_out_param::val_int()
-{
-  assert(0);
-  return 0;
-}
-
-
-String* Item_user_var_as_out_param::val_str(String *str __attribute__((unused)))
-{
-  assert(0);
-  return 0;
-}
-
-
-my_decimal* Item_user_var_as_out_param::val_decimal(my_decimal *decimal_buffer __attribute__((unused)))
-{
-  assert(0);
-  return 0;
-}
-
-
-void Item_user_var_as_out_param::print(String *str,
-                                       enum_query_type query_type __attribute__((unused)))
-{
-  str->append('@');
-  str->append(name.str,name.length);
-}
-
-/***************************************************************************
-  System variables
-****************************************************************************/
-
-/**
-  Return value of an system variable base[.name] as a constant item.
-
-  @param session			Thread handler
-  @param var_type		global / session
-  @param name		        Name of base or system variable
-  @param component		Component.
-
-  @note
-    If component.str = 0 then the variable name is in 'name'
-
-  @return
-    - 0  : error
-    - #  : constant item
-*/

=== modified file 'drizzled/item_func.h'
--- drizzled/item_func.h	2008-10-29 02:36:17 +0000
+++ drizzled/item_func.h	2008-10-30 21:40:56 +0000
@@ -63,6 +63,7 @@
 #include <drizzled/functions/real.h>
 #include <drizzled/functions/rollup_const.h>
 #include <drizzled/functions/row_count.h>
+#include <drizzled/functions/set_user_var.h>
 #include <drizzled/functions/dec.h>
 #include <drizzled/functions/int_val.h>
 #include <drizzled/functions/acos.h>
@@ -88,95 +89,10 @@
 #include <drizzled/functions/sign.h>
 #include <drizzled/functions/signed.h>
 #include <drizzled/functions/tan.h>
+#include <drizzled/functions/update_hash.h>
+#include <drizzled/functions/user_var_as_out_param.h>
 #include <drizzled/functions/unsigned.h>
 
-/* Handling of user definable variables */
-
-class user_var_entry;
-
-class Item_func_set_user_var :public Item_func
-{
-  enum Item_result cached_result_type;
-  user_var_entry *entry;
-  char buffer[MAX_FIELD_WIDTH];
-  String value;
-  my_decimal decimal_buff;
-  bool null_item;
-  union
-  {
-    int64_t vint;
-    double vreal;
-    String *vstr;
-    my_decimal *vdec;
-  } save_result;
-
-public:
-  LEX_STRING name; // keep it public
-  Item_func_set_user_var(LEX_STRING a,Item *b)
-    :Item_func(b), cached_result_type(INT_RESULT), name(a)
-  {}
-  enum Functype functype() const { return SUSERVAR_FUNC; }
-  double val_real();
-  int64_t val_int();
-  String *val_str(String *str);
-  my_decimal *val_decimal(my_decimal *);
-  double val_result();
-  int64_t val_int_result();
-  String *str_result(String *str);
-  my_decimal *val_decimal_result(my_decimal *);
-  bool update_hash(void *ptr, uint32_t length, enum Item_result type,
-  		   const CHARSET_INFO * const cs, Derivation dv, bool unsigned_arg);
-  bool send(Protocol *protocol, String *str_arg);
-  void make_field(Send_field *tmp_field);
-  bool check(bool use_result_field);
-  bool update();
-  enum Item_result result_type () const { return cached_result_type; }
-  bool fix_fields(Session *session, Item **ref);
-  void fix_length_and_dec();
-  virtual void print(String *str, enum_query_type query_type);
-  void print_as_stmt(String *str, enum_query_type query_type);
-  const char *func_name() const { return "set_user_var"; }
-  int save_in_field(Field *field, bool no_conversions,
-                    bool can_use_result_field);
-  int save_in_field(Field *field, bool no_conversions)
-  {
-    return save_in_field(field, no_conversions, 1);
-  }
-  void save_org_in_field(Field *field) { (void)save_in_field(field, 1, 0); }
-  bool register_field_in_read_map(unsigned char *arg);
-  bool register_field_in_bitmap(unsigned char *arg);
-};
-
-
-/*
-  This item represents user variable used as out parameter (e.g in LOAD DATA),
-  and it is supposed to be used only for this purprose. So it is simplified
-  a lot. Actually you should never obtain its value.
-
-  The only two reasons for this thing being an Item is possibility to store it
-  in List<Item> and desire to place this code somewhere near other functions
-  working with user variables.
-*/
-class Item_user_var_as_out_param :public Item
-{
-  LEX_STRING name;
-  user_var_entry *entry;
-public:
-  Item_user_var_as_out_param(LEX_STRING a) : name(a) {}
-  /* We should return something different from FIELD_ITEM here */
-  enum Type type() const { return STRING_ITEM;}
-  double val_real();
-  int64_t val_int();
-  String *val_str(String *str);
-  my_decimal *val_decimal(my_decimal *decimal_buffer);
-  /* fix_fields() binds variable name with its entry structure */
-  bool fix_fields(Session *session, Item **ref);
-  virtual void print(String *str, enum_query_type query_type);
-  void set_null_value(const CHARSET_INFO * const cs);
-  void set_value(const char *str, uint32_t length, const CHARSET_INFO * const cs);
-};
-
-
 /* For type casts */
 
 enum Cast_target

